-- GP7 Engine script
-- DRB APR 10

-- Includes code for controlling variable exhaust emission.
-- And processing rpm and rpmdelta for leading->trailing engines.
-- Note that all this code is Railworks specific!

IDLERPM = 315
FULLRPM = 900

DRBRPM_ID = 1709
DRBRPMD_ID = 1710


function Initialise ()

-- Turn off "surge" at start.
	Call( "Exhaust1a:SetEmitterActive", 0 )
	Call( "Exhaust2a:SetEmitterActive", 0 )

	gPrevRPM = 0
	gPrevRPMDelta = 0

	Call( "BeginUpdate" )

end


function OnControlValueChange ( name, index, value )

	if Call( "*:ControlExists", name, index ) then

		Call( "*:SetControlValue", name, index, value );

	end

end


function Update ( time )

-- Get rpm values for this vehicle.
	rpm = Call( "*:GetControlValue", "RPM", 0 )
	rpm_change = Call( "*:GetControlValue", "RPMDelta", 0 )

-- "surge" when rpm increases.
	if ( rpm_change > 0.01 ) then
		Call( "Exhaust1a:SetEmitterActive", 1 )
		Call( "Exhaust2a:SetEmitterActive", 1 )
	else
		Call( "Exhaust1a:SetEmitterActive", 0 )
		Call( "Exhaust2a:SetEmitterActive", 0 )
	end

-- Adjust basic exhaust intensity and colour based on rpm.
-- Rate of 0.1 for idle, 0 for full revs.
-- Colour multiplier (lightener) of 5 for idle, 1 for full revs.

	exhaustrate = 0.1 - 0.1*(rpm-IDLERPM)/(FULLRPM-IDLERPM)
	exh_col = 5 - 5*(rpm-IDLERPM)/(FULLRPM-IDLERPM)
	Call( "Exhaust1:SetEmitterRate", exhaustrate )
	Call( "Exhaust1:SetEmitterColour", exh_col, exh_col, exh_col )
	Call( "Exhaust2:SetEmitterRate", exhaustrate )
	Call( "Exhaust2:SetEmitterColour", exh_col, exh_col, exh_col )


-- If this is a player train and this vehicle is the driven one, then send
-- RPM values in both directions along train. Note the Driven control is set
-- by the simulation script, which only runs on the vehicle the player is driving.
-- Only send messages if values have changed.

	if ( Call( "GetIsPlayer" ) == 1 ) then

		if ( Call( "*:GetControlValue", "Driven", 0 ) == 1 ) then
			if ( rpm ~= gPrevRPM ) then
				Call( "SendConsistMessage", DRBRPM_ID, rpm, 1 )
				Call( "SendConsistMessage", DRBRPM_ID, rpm, 0 )
			end
			if ( rpm_change ~= gPrevRPMDelta ) then
				Call( "SendConsistMessage", DRBRPMD_ID, rpm_change, 1 )
				Call( "SendConsistMessage", DRBRPMD_ID, rpm_change, 0 )
			end
		end

		gPrevRPM = rpm
		gPrevRPMDelta = rpm_change

	end

end


-- Core sim function executed if a consist message is received.

function OnConsistMessage ( msg, argument, direction )

-- If this is not the driven vehicle then update the controls for RPM values with
-- the supplied data.

	if ( Call( "*:GetControlValue", "Driven", 0 ) == 0 ) then

		if ( msg == DRBRPM_ID ) then
			Call( "*:SetControlValue", "RPM", 0, argument )
		elseif ( msg == DRBRPMD_ID ) then
			Call( "*:SetControlValue", "RPMDelta", 0, argument )
		end

-- Pass message along in same direction.
		Call( "SendConsistMessage", msg, argument, direction )

	end

--	Print(( "DRB: msg= " .. msg .. ", arg= " .. argument .. ", dir= " .. direction ))

end