------------------------------------------------------------
-- Simulation file for the GP7
------------------------------------------------------------
-- DRB APR 10
------------------------------------------------------------
-- Setup
------------------------------------------------------------
-- Called when the engine script is initialised
------------------------------------------------------------

function Setup ()

	gPCStime = 0
	gLocoBrkSet = 0
	gParallel = 0

end

-- Constants

-- PCS will trip 4 mph above this speed.
	MAXLOCOSPD = 65
-- Speed for upwards series->parallel transistion.
	TRANSSPD = 25

------------------------------------------------------------
-- Update
------------------------------------------------------------
-- Called every frame to update the simulation
------------------------------------------------------------
-- Parameters:
--	inteval = time since last update
------------------------------------------------------------

function Update (inteval)

-- Get engine and core sim parameters.
	CurrentTime = Call( "*:GetSimulationTime", 0 )
	CurrentSpeed = Call( "*:GetControlValue", "SpeedometerMPH", 0 )
	BrakeHdl = Call( "*:GetControlValue", "trainbrkhdl", 0 )
	LocoHdl = Call( "*:GetControlValue", "locobrkhdl", 0 )
	LocoPressure = Call( "*:GetControlValue", "LocoBrakeCylinderPressurePSI", 0 )
	TrainPressure = Call( "*:GetControlValue", "TrainBrakeCylinderPressurePSI", 0 )
	BrakePipe = Call( "*:GetControlValue", "AirBrakePipePressurePSI", 0 )
	ThrottleHdl = Call( "*:GetControlValue", "throttlehdl", 0 )
	ReverseHdl = Call( "*:GetControlValue", "reversehdl", 0 )
	GenField = Call( "*:GetControlValue", "genfieldbut", 0 )
	FuelPump = Call( "*:GetControlValue", "fuelpumpbut", 0 )
	EngCntrl = Call( "*:GetControlValue", "enginecntrlbut", 0 )
	PCSstate = Call( "*:GetControlValue", "PCSopen", 0 )
	EmergSet = Call( "*:GetControlValue", "EmergencyBrake", 0 )
	Lights = Call( "*:GetControlValue", "headlever", 0 )

-- 6BL train brake handle.

	if ( BrakeHdl == 1 ) or ( PCSstate == 3 ) then
		BrakeSet = 1
		gLocoBrkSet = 1
	elseif ( BrakeHdl < 1 ) and ( BrakeHdl > 0.81 ) then
		BrakeSet = 0.6
	elseif ( BrakeHdl <= 0.81 ) and ( BrakeHdl > 0.6101 ) then
		BrakeSet = BrakeHdl
		if ( TrainPressure / 65 ) > gLocoBrkSet then
			gLocoBrkSet = ( TrainPressure / 65 )
		end
	elseif ( BrakeHdl <= 0.6101 ) and ( BrakeHdl > 0.38 ) then
		BrakeSet = 0.6101
		if ( TrainPressure / 65 ) > gLocoBrkSet then
			gLocoBrkSet = ( TrainPressure / 65 )
		end
	elseif ( BrakeHdl > 0.379 ) and ( BrakeHdl < 0.381 ) then
		BrakeSet = BrakeHdl
	else
		BrakeSet = BrakeHdl
		gLocoBrkSet = 0
	end

	if ( EmergSet == 0 ) then
		Call( "*:SetControlValue", "TrainBrakeControl", 0, BrakeSet )
	end

-- 6BL loco brake.

	if ( gLocoBrkSet > LocoHdl ) then
		Call( "*:SetControlValue", "EngineBrakeControl", 0, gLocoBrkSet )
	else
		Call( "*:SetControlValue", "EngineBrakeControl", 0, LocoHdl )
	end

-- Power and reverser. Including simulation of transistion.

	if ( EngCntrl == 1 ) and ( PCSstate < 3 ) then
		if ( ThrottleHdl == 0 ) then
			Call( "*:SetControlValue", "Regulator", 0, 0 )
			gParallel = 0
		elseif ( CurrentSpeed > TRANSSPD ) and ( gParallel == 0 ) then
			Call( "*:SetControlValue", "Regulator", 0, 0.01 )
			gParallel = 1
		elseif (( CurrentSpeed + 5 ) < TRANSSPD ) and ( gParallel == 1 ) then
			Call( "*:SetControlValue", "Regulator", 0, 0.01 )
			gParallel = 0
		else
			Call( "*:SetControlValue", "Regulator", 0, ThrottleHdl )
		end
	else
		Call( "*:SetControlValue", "Regulator", 0, 0 )
	end

	if ( GenField == 1 ) and ( PCSstate < 3 ) then
		Call( "*:SetControlValue", "Reverser", 0, ReverseHdl )
	else
		Call( "*:SetControlValue", "Reverser", 0, 0 )
	end

-- Pneumatic control switch.

	if ( CurrentSpeed > ( MAXLOCOSPD + 4 )) then
		if ( gPCStime == 0 ) then
			Call( "*:SetControlValue", "PCSopen", 0, 2 )
			gPCStime = CurrentTime
		elseif ( CurrentTime > gPCStime + 5 ) then
			Call( "*:SetControlValue", "PCSopen", 0, 3 )
			Call( "*:SetControlValue", "EmergencyBrake", 0, 1 )
		end
	elseif ( CurrentSpeed > MAXLOCOSPD ) and ( PCSstate < 3 ) then
		Call( "*:SetControlValue", "PCSopen", 0, 1 )
		gPCStime = 0
	elseif ( CurrentSpeed <= MAXLOCOSPD ) and ( PCSstate < 3 ) then
		Call( "*:SetControlValue", "PCSopen", 0, 0 )
		gPCStime = 0
	end

	if ( EmergSet == 1 ) or (( BrakeHdl == 1 ) and ( ReverseHdl ~= 0 )) then
		Call( "*:SetControlValue", "PCSopen", 0, 3 )
	end

	if ( EmergSet == 0 ) and ( ThrottleHdl == 0 ) and ( BrakeHdl > 0.59 ) and ( BrakeHdl < 0.62 ) then
		Call( "*:SetControlValue", "PCSopen", 0, 0 )
	end

-- Headlights.

	if ( Lights == -1 ) then
		Call( "*:SetControlValue", "Headlights", 0, 1 )
	elseif ( Lights == 1 ) then
		Call( "*:SetControlValue", "Headlights", 0, 2 )
	else
		Call( "*:SetControlValue", "Headlights", 0, 0 )
	end


-- Begin Railworks stuff.

-- If simulation is running this must be player vehicle, so set Driven
-- control to indicate this.

	Call( "*:SetControlValue", "Driven", 0, 1 )

end

